package com.pixelmintdrop.model

/**
 * Represents a game piece
 */
enum class GamePieceType {
    I, J, L, O, S, T, Z
}

class GamePiece(val type: GamePieceType) {
    private var rotation = 0

    // Each piece has 4 rotations (0, 90, 180, 270 degrees)
    private val blocks: Array<Array<BooleanArray>> = getBlocks(type)

    /**
     * Get the current shape of the piece based on rotation
     */
    fun getShape(): Array<BooleanArray> = blocks[rotation]

    /**
     * Get the width of the current piece shape
     */
    fun getWidth(): Int = blocks[rotation][0].size

    /**
     * Get the height of the current piece shape
     */
    fun getHeight(): Int = blocks[rotation].size

    /**
     * Rotate the piece clockwise
     */
    fun rotateClockwise() {
        rotation = (rotation + 1) % 4
    }

    /**
     * Rotate the piece counter-clockwise
     */
    fun rotateCounterClockwise() {
        rotation = (rotation + 3) % 4
    }

    /**
     * Check if the piece's block exists at the given coordinates
     */
    fun hasBlock(x: Int, y: Int): Boolean {
        if (x < 0 || x >= getWidth() || y < 0 || y >= getHeight()) return false
        return blocks[rotation][y][x]
    }

    /**
     * Get the block patterns for each piece type and all its rotations
     */
    private fun getBlocks(type: GamePieceType): Array<Array<BooleanArray>> {
        return when (type) {
            GamePieceType.I -> arrayOf(
                arrayOf(
                    booleanArrayOf(false, false, false, false),
                    booleanArrayOf(true, true, true, true),
                    booleanArrayOf(false, false, false, false),
                    booleanArrayOf(false, false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, true, false),
                    booleanArrayOf(false, false, true, false),
                    booleanArrayOf(false, false, true, false),
                    booleanArrayOf(false, false, true, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false, false),
                    booleanArrayOf(false, false, false, false),
                    booleanArrayOf(true, true, true, true),
                    booleanArrayOf(false, false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false, false),
                    booleanArrayOf(false, true, false, false),
                    booleanArrayOf(false, true, false, false),
                    booleanArrayOf(false, true, false, false)
                )
            ),
            GamePieceType.J -> arrayOf(
                arrayOf(
                    booleanArrayOf(true, false, false),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(false, false, true)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(true, true, false)
                )
            ),
            GamePieceType.L -> arrayOf(
                arrayOf(
                    booleanArrayOf(false, false, true),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, true)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(true, false, false)
                ),
                arrayOf(
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, false)
                )
            ),
            GamePieceType.O -> arrayOf(
                arrayOf(
                    booleanArrayOf(true, true),
                    booleanArrayOf(true, true)
                ),
                arrayOf(
                    booleanArrayOf(true, true),
                    booleanArrayOf(true, true)
                ),
                arrayOf(
                    booleanArrayOf(true, true),
                    booleanArrayOf(true, true)
                ),
                arrayOf(
                    booleanArrayOf(true, true),
                    booleanArrayOf(true, true)
                )
            ),
            GamePieceType.S -> arrayOf(
                arrayOf(
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(false, false, true)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false),
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(true, true, false)
                ),
                arrayOf(
                    booleanArrayOf(true, false, false),
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, true, false)
                )
            ),
            GamePieceType.T -> arrayOf(
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(false, true, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false),
                    booleanArrayOf(true, true, true),
                    booleanArrayOf(false, true, false)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, true, false)
                )
            ),
            GamePieceType.Z -> arrayOf(
                arrayOf(
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(false, false, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, true),
                    booleanArrayOf(false, true, true),
                    booleanArrayOf(false, true, false)
                ),
                arrayOf(
                    booleanArrayOf(false, false, false),
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(false, true, true)
                ),
                arrayOf(
                    booleanArrayOf(false, true, false),
                    booleanArrayOf(true, true, false),
                    booleanArrayOf(true, false, false)
                )
            )
        }
    }
} 